/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicInteger;

import com.redhat.rhevm.api.model.BaseResource;

import com.redhat.rhevm.api.common.resource.AbstractActionableResource;
import com.redhat.rhevm.api.common.resource.UriInfoProvider;


public abstract class AbstractMockResource<R extends BaseResource> extends AbstractActionableResource<R> {

    private static Map<Class<?>, AtomicInteger> counters = new HashMap<Class<?>, AtomicInteger>();

    public AbstractMockResource(String id) {
        super(id);
    }

    public AbstractMockResource(String id, UriInfoProvider uriProvider) {
        super(id);
        this.uriProvider = uriProvider;
    }

    public AbstractMockResource(String id, Executor executor) {
        super(id, executor);
    }

    public AbstractMockResource(String id, Executor executor, UriInfoProvider uriProvider) {
        super(id, executor, uriProvider);
    }

    public static String allocateId(Class<?> clazz) {
        return Integer.toString(getCounter(clazz));
    }

    public synchronized R getModel() {
        return model;
    }

    /**
     * A per-generic-instantiation map of counters.
     *
     * @param   clz the generic type parameter
     * @return  the counter value for that type
     */
    private static int getCounter(Class<?> clz) {
        AtomicInteger i = counters.get(clz);
        if (i == null) {
            i = new AtomicInteger();
            counters.put(clz, i);
        }
        return i.incrementAndGet();
    }
}
