/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Role;
import com.redhat.rhevm.api.model.Roles;
import com.redhat.rhevm.api.resource.RoleResource;
import com.redhat.rhevm.api.resource.RolesResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockRolesResource extends AbstractMockQueryableResource<Role> implements RolesResource {

    private static Map<String, MockRoleResource> roles =
        Collections.synchronizedMap(new HashMap<String, MockRoleResource>());

    public MockRolesResource() {
        super(new SimpleQueryEvaluator<Role>());
    }

    public void populate() {
        synchronized (roles) {
            while (roles.size() < 2) {
                MockRoleResource resource = new MockRoleResource(allocateId(Role.class), getExecutor(), this);
                Role role = resource.getModel();
                role.setName("role" + resource.getModel().getId());
                role.setDescription("test role");
                roles.put(role.getId(), resource);
            }
        }
    }

    @Override
    public Roles list() {
        Roles ret = new Roles();

        for (MockRoleResource role : roles.values()) {
            UriInfo info = getUriInfo();
            if (filter(role.getModel(), info, Role.class)) {
                ret.getRoles().add(role.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(Role role) {
        MockRoleResource resource = new MockRoleResource(allocateId(Role.class), getExecutor(), this);

        resource.updateModel(role);

        String id = resource.getId();
        roles.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        role = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(role).build();
    }

    @Override
    public void remove(String id) {
        roles.remove(id);
    }

    @Override
    public RoleResource getRoleSubResource(String id) {
         return roles.get(id);
    }
}
