/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Cluster;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.VmPool;
import com.redhat.rhevm.api.model.VmPools;
import com.redhat.rhevm.api.resource.VmPoolResource;
import com.redhat.rhevm.api.resource.VmPoolsResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockVmPoolsResource extends AbstractMockQueryableResource<VmPool> implements VmPoolsResource {

    private static Map<String, MockVmPoolResource> pools =
        Collections.synchronizedMap(new HashMap<String, MockVmPoolResource>());

    public MockVmPoolsResource() {
        super(new SimpleQueryEvaluator<VmPool>());
    }

    public void populate() {
        synchronized (pools) {
            while (pools.size() < 3) {
                MockVmPoolResource resource = new MockVmPoolResource(allocateId(VmPool.class), getExecutor(), this);
                VmPool pool = resource.getModel();
                pool.setName("pool" + resource.getModel().getId());
                pool.setDescription("test pool " + resource.getModel().getId());
                Template template = new Template();
                template.setId(resource.getModel().getId());
                resource.getModel().setTemplate(template);
                Cluster cluster = new Cluster();
                cluster.setId(resource.getModel().getId());
                resource.getModel().setCluster(cluster);
                pools.put(pool.getId(), resource);
            }
        }
    }

    @Override
    public VmPools list() {
        VmPools ret = new VmPools();

        for (MockVmPoolResource pool : pools.values()) {
            if (filter(pool.getModel(), getUriInfo(), VmPool.class)) {
                ret.getVmPools().add(pool.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(VmPool pool) {
        MockVmPoolResource resource = new MockVmPoolResource(allocateId(VmPool.class), getExecutor(), this);

        resource.updateModel(pool);

        String id = resource.getId();
        pools.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);
        pool = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(pool).build();
    }

    @Override
    public void remove(String id) {
        pools.remove(id);
    }

    @Override
    public VmPoolResource getVmPoolSubResource(String id) {
        return pools.get(id);
    }
}
