/*
* Copyright © 2010 Red Hat, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*           http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package com.redhat.rhevm.api.mock.resource;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

import com.redhat.rhevm.api.mock.util.SimpleQueryEvaluator;
import com.redhat.rhevm.api.model.Action;
import com.redhat.rhevm.api.model.Template;
import com.redhat.rhevm.api.model.VM;
import com.redhat.rhevm.api.model.VMs;
import com.redhat.rhevm.api.model.VmType;
import com.redhat.rhevm.api.resource.VmResource;
import com.redhat.rhevm.api.resource.VmsResource;

import static com.redhat.rhevm.api.mock.resource.AbstractMockResource.allocateId;


public class MockVmsResource extends AbstractMockQueryableResource<VM> implements VmsResource {

    private static Map<String, MockVmResource> vms =
        Collections.synchronizedMap(new HashMap<String, MockVmResource>());

    public MockVmsResource() {
        super(new SimpleQueryEvaluator<VM>());
    }

    public void populate() {
        synchronized (vms) {
            while (vms.size() < 10) {
                MockVmResource vmResource = new MockVmResource(allocateId(VM.class), getExecutor(), this);
                vmResource.getModel().setName("vm" + vmResource.getModel().getId());
                vmResource.getModel().setType(VmType.SERVER.value());
                vmResource.getModel().setTemplate(new Template());
                vmResource.getModel().getTemplate().setId(allocateId(Template.class));
                vms.put(vmResource.getModel().getId(), vmResource);
            }
        }
    }

    @Override
    public VMs list() {
        VMs ret = new VMs();

        for (MockVmResource vm : vms.values()) {
            if (filter(vm.getModel(), getUriInfo(), VM.class)) {
                ret.getVMs().add(vm.addLinks());
            }
        }

        return ret;
    }

    @Override
    public Response add(VM vm) {
        MockVmResource resource = new MockVmResource(allocateId(VM.class), getExecutor(), this);

        resource.updateModel(vm);

        String id = resource.getId();
        vms.put(id, resource);

        UriBuilder uriBuilder = getUriInfo().getAbsolutePathBuilder().path(id);

        vm = resource.addLinks();

        return Response.created(uriBuilder.build()).entity(vm).build();
    }

    @Override
    public void remove(String id) {
        vms.remove(id);
    }

    @Override
    public void remove(String id, Action action) {
        vms.remove(id);
    }

    @Override
    public VmResource getVmSubResource(String id) {
        return vms.get(id);
    }
}
